lopts = {}
lopts_default = {}
lopts['ru_RU'] = ['hyphen', 'comma', 'multiword', 'together', 'common', 'space', 'abbreviation', 'dup', 'numsep', 'typographica', 'quotation']
lopts_default['ru_RU'] = ['hyphen', 'comma', 'multiword', 'together', 'common', 'space', 'abbreviation', 'dup', 'typographica']
